#include <gtk/gtk.h>


void
on_add_button_clicked                  (GtkButton       *button,
                                        gpointer         user_data);

void
on_remove_button_clicked               (GtkButton       *button,
                                        gpointer         user_data);

void
on_cancel_button_clicked               (GtkButton       *button,
                                        gpointer         user_data);

void
on_save_button_clicked                 (GtkButton       *button,
                                        gpointer         user_data);

void
on_edit_button_clicked                 (GtkButton       *button,
                                        gpointer         user_data);

void
on_edit_cancel_button_clicked          (GtkButton       *button,
                                        gpointer         user_data);

void
on_edit_ok_button_clicked              (GtkButton       *button,
                                        gpointer         user_data);

void
on_shellexec_conf_dialog_destroy       (GObject       *object,
                                        gpointer         user_data);
